# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-05-30 08:20
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0027_scheduling_class'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='collection',
            name='public_processes',
        ),
        migrations.RemoveField(
            model_name='entity',
            name='public_processes',
        ),
    ]
