# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-27 13:13
from __future__ import unicode_literals

from django.db import migrations
import resolwe.flow.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0029_data_checksum_index'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collection',
            name='slug',
            field=resolwe.flow.models.fields.ResolweSlugField(max_length=100, populate_from='name', unique_with=('version',)),
        ),
        migrations.AlterField(
            model_name='data',
            name='slug',
            field=resolwe.flow.models.fields.ResolweSlugField(max_length=100, populate_from='name', unique_with=('version',)),
        ),
        migrations.AlterField(
            model_name='descriptorschema',
            name='slug',
            field=resolwe.flow.models.fields.ResolweSlugField(max_length=100, populate_from='name', unique_with=('version',)),
        ),
        migrations.AlterField(
            model_name='entity',
            name='slug',
            field=resolwe.flow.models.fields.ResolweSlugField(max_length=100, populate_from='name', unique_with=('version',)),
        ),
        migrations.AlterField(
            model_name='process',
            name='slug',
            field=resolwe.flow.models.fields.ResolweSlugField(max_length=100, populate_from='name', unique_with=('version',)),
        ),
        migrations.AlterField(
            model_name='relation',
            name='slug',
            field=resolwe.flow.models.fields.ResolweSlugField(max_length=100, populate_from='name', unique_with=('version',)),
        ),
        migrations.AlterField(
            model_name='storage',
            name='slug',
            field=resolwe.flow.models.fields.ResolweSlugField(max_length=100, populate_from='name', unique_with=('version',)),
        ),
    ]
