# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-02 08:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0018_remove_triggers'),
    ]

    operations = [
        migrations.AddField(
            model_name='data',
            name='descriptor_dirty',
            field=models.BooleanField(default=False),
        ),
    ]
