# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-08 13:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0026_tags'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='process',
            name='priority',
        ),
        migrations.AddField(
            model_name='process',
            name='scheduling_class',
            field=models.CharField(choices=[('IN', 'Interactive'), ('BA', 'Batch')], default='BA', max_length=2),
        ),
    ]
