# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-05-26 06:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0006_data_named_by_user'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='collection',
            options={'default_permissions': (), 'permissions': (('view_collection', 'Can view collection'), ('edit_collection', 'Can edit collection'), ('share_collection', 'Can share collection'), ('download_collection', 'Can download files from collection'), ('add_collection', 'Can add data objects to collection'), ('owner_collection', 'Is owner of the collection'))},
        ),
        migrations.AlterModelOptions(
            name='data',
            options={'default_permissions': (), 'permissions': (('view_data', 'Can view data'), ('edit_data', 'Can edit data'), ('share_data', 'Can share data'), ('download_data', 'Can download files from data'), ('owner_data', 'Is owner of the data'))},
        ),
        migrations.AlterModelOptions(
            name='descriptorschema',
            options={'default_permissions': (), 'permissions': (('view_descriptorschema', 'Can view descriptor schema'), ('edit_descriptorschema', 'Can edit descriptor schema'), ('share_descriptorschema', 'Can share descriptor schema'), ('owner_descriptorschema', 'Is owner of the description schema'))},
        ),
        migrations.AlterModelOptions(
            name='process',
            options={'default_permissions': (), 'permissions': (('view_process', 'Can view process'), ('share_process', 'Can share process'), ('owner_process', 'Is owner of the process'))},
        ),
        migrations.AlterModelOptions(
            name='trigger',
            options={'default_permissions': (), 'permissions': (('view_trigger', 'Can view trigger'), ('edit_trigger', 'Can edit trigger'), ('share_trigger', 'Can share trigger'), ('owner_trigger', 'Is owner of the trigger'))},
        ),
    ]
