# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-31 07:23
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0017_update_checksum'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='trigger',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='trigger',
            name='collection',
        ),
        migrations.RemoveField(
            model_name='trigger',
            name='contributor',
        ),
        migrations.RemoveField(
            model_name='trigger',
            name='process',
        ),
        migrations.DeleteModel(
            name='Trigger',
        ),
    ]
